<?php
namespace PME\Controllers\Custo;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class DesvioConsumoController extends \Zeedhi\Framework\Controller\Report {
    protected $birtStrategy;
    protected $newCode;
    protected $lista;
    protected $connection;
    protected $ConsumoPlanejadoRealizadoController;
    protected $utilities;

    public function __construct(ReportService $reportService, $birtStrategy, $newCode, $lista, $connection, $ConsumoPlanejadoRealizadoController, $utilities) {
      parent::__construct($reportService);
      $this->birtStrategy  = $birtStrategy;
      $this->newCode       = $newCode;
      $this->lista         = $lista;
      $this->connection    = $connection;
      $this->ConsumoPlanejadoRealizadoController = $ConsumoPlanejadoRealizadoController;
      $this->utilities     = $utilities;
    }

    protected $reportMapping = array (
      'PME01600' => 
      array (
        'strategy' => 'BIRT',
        'parameterMapping' => 
        array (
          "CDTIPOCUSTO"   => "P_NRTIPOCUSTO",
          "TAMACNIVEL"    => "P_NRTAMACNIVEL",
          "ORDENA"        => "P_NRTAMCODIGO",
          "CDFILCENTPME"  => "P_CDFILCENTPME",
          "NRCONSPLAN"    => "P_NRCONSPLAN",
          "LISTASERV"     => "P_NRLISTASERVICO", 
          "PRODINICIAL"   => "P_CDPRODINIC",
          "PRODFINAL"     => "P_CDPRODFINA",
          "ESTOCAVEIS"    => "P_IDESTOCAVEIS",
          "DTMESINIC"     => "P_DTMESINIC",
          "DTMESFINA"     => "P_DTMESFINA",
          "NAOESTOCAVEIS" => "P_IDNESTOCAVEIS",
          "DTDE"          => "P_DTINICIAL",
          "DTATE"         => "P_DTFINAL",
          "NMTIPOCUSTO"   => "P_NMTIPOCUSTO",
          "CDPICTPROD"    => "P_CDPICTPROD",
          "DIFQTD"        => "P_IDDIFERENCA",
          "AGRUPMASK"     => "P_MASCAGRUP",
          "NRORG"         => "P_NRORG"
        )
      ),
      'PME01601' => 
      array (
        'strategy' => 'BIRT',
        'parameterMapping' => 
        array (
          "CDTIPOCUSTO"   => "P_NRTIPOCUSTO",
          "TAMACNIVEL"    => "P_NRTAMACNIVEL",
          "ORDENA"        => "P_NRTAMCODIGO",
          "DTDE"          => "P_DTINICIAL",
          "DTATE"         => "P_DTFINAL",
          "LISTASERV"     => "P_NRLISTASERVICO", 
          "CDFILCENTPME"  => "P_CDFILCENTPME",
          "NRCONSPLAN"    => "P_NRCONSPLAN",
          "PRODINICIAL"   => "P_CDPRODINIC",
          "PRODFINAL"     => "P_CDPRODFINA",
          "ESTOCAVEIS"    => "P_IDESTOCAVEIS",
          "DTMESINIC"     => "P_DTMESINIC",
          "DTMESFINA"     => "P_DTMESFINA",
          "NAOESTOCAVEIS" => "P_IDNESTOCAVEIS",
          "DIFQTD"        => "P_IDDIFERENCA",
          "AGRUPMASK"     => "P_MASCAGRUP",
          "CDPICTPROD"    => "P_CDPICTPROD",
          "NMTIPOCUSTO"   => "P_NMTIPOCUSTO",
          "NRORG"         => "P_NRORG"
        )
      )
    );
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
      $rows = $request->getRow();
      
      $nrorg = $this->utilities->getNrOrg();
      $rows['NRORG'] = $nrorg;
      $rows['NRCONSPLAN'] = $this->newCode->getCodeByTableName('CONSPLAN'.$rows['CDFILCENTPME'], $nrorg);
      
      $newRow = $this->ConsumoPlanejadoRealizadoController->procedure($rows);

      /* if ($rows['GRUPFILI'] == NULL) {
        if (empty($rows['CDFILIAL'])) {
          $rows['CDFILIAL'] = $this->ConsumoPlanejadoRealizadoController->getUnidades($rows);
        } else {
          $rows['CDFILIAL'] = is_array($rows['CDFILIAL']) ? join("_", $rows['CDFILIAL']) : $rows['CDFILIAL'];
        }
      } else {
        if (empty($rows['CDFILIAL'])) $rows['CDFILIAL'] = $rows['CDFILIAL'] = $this->ConsumoPlanejadoRealizadoController->getUnidadesGrupo($rows, $nrorg);  
        else $rows['CDFILIAL'] = is_array($rows['CDFILIAL']) ? join("_", $rows['CDFILIAL']) : $rows['CDFILIAL'];
      }*/
      
      $rows['UNIDADE'] = $newRow['UNIDADE'];
      $rows['SERVICO'] = $newRow['SERVICO'];
      
      $rows['UNIDADE'] = explode('_', $rows['UNIDADE']);
      $rows['LISTASERV'] = $this->geraListaServico($rows, $nrorg);

      if($rows['ORDENA'] == 0) $rows['TAMCODIGO'] = 10;
      else $rows['TAMCODIGO'] = 0;
      
      $cdprodIni = isset($rows['PRODINICIAL']['CDARVPROD']) ? $rows['PRODINICIAL']['CDARVPROD'] : 0;
      if(!(isset($rows['PRODFINAL']['CDARVPROD']))) $cdprodFin = 'zzzzzzzzzzz';
      else $cdprodFin = str_pad($rows['PRODFINAL']['CDARVPROD'], 10, 'z');
  
      $rows['PRODINICIAL'] = $cdprodIni;
      $rows['PRODFINAL'] = $cdprodFin;
      
      $this->birtStrategy->setReportLocale($this->utilities->getLanguage());
      $this->birtStrategy->setReportFormat(strtolower($rows['reportFormat']));
    }
    
  private function geraListaServico($rows, $nrorg) {
    try {
      $nrlista = $this->newCode->getCodeByTableName('LISTA_SERVICO', $nrorg);
      
      $sql = "INSERT INTO LISTA_SERVICO (NRLISTA, CDFILIAL, CDSERVFILI, NMSERVICO, CDGRUPSERV, NRORG) VALUES (:NRLISTA, :CDFILIAL, :CDSERVFILI, :NMSERVICO, :CDGRUPSERV, :NRORG)";
      $servicosFiliais = $rows['SERVICO'];
      if (!is_array($servicosFiliais)) $servicosFiliais = $rows['CDSERVFILI'];
      if (!$servicosFiliais) {
        $query = "SELECT DISTINCT S.CDGRUPSERV,
                          S.CDFILIAL,
                          S.CDSERVFILI,
                          S.NMSERVICO,
                          ( S.CDFILIAL
                            || S.CDSERVFILI ) AS FILISERV,
                          S.NRORG
            FROM FILIAL      FI, SERVICO     S
           WHERE ( FI.CDFILCENTPME = :CDFILCENTPME )
             AND ( FI.CDFILIAL = S.CDFILIAL )
             AND ( ( INSTR(:CDFILIAL, S.CDFILIAL) <> 0 ) )
             AND ( ( INSTR(:CDGRUPSERV, S.CDGRUPSERV) <> 0 )
              OR ( :CDGRUPSERV = 'T' ) )
          ORDER BY S.CDSERVFILI";
          
        $arrayResult = [];

        $unidade = join("_", $rows['UNIDADE']);
        for ($i = 0; $i <= strlen($unidade); $i+=3000) {
          $unid = substr($unidade, $i, 3000);
          $params = array(
            "CDFILCENTPME" => $rows["CDFILCENTPME"],
            "CDFILIAL"     => $unid,
            "CDGRUPSERV"   => $rows['GRUPOSERVICO']
          );
          $stmt = $this->connection->prepare($query);
      	  $stmt->execute($params);
          $result = $stmt->fetchAll();
          
          foreach($result as $posicao) {
            $arrayResult = $posicao;
          }
        }
        
        // $servicosFiliais = $this->ConsumoPlanejadoRealizadoController->getUnidServ($rows, false);
        $servicosFiliais = $arrayResult;
      }
      $stmt = $this->connection->prepare($sql);
  
      foreach ($servicosFiliais as $servicoFilial) {
        $stmt->execute(
          array(
            "NRLISTA"    => $nrlista,
            "CDFILIAL"   => $servicoFilial['CDFILIAL'],
            "NRORG"      => $rows['NRORG'],
            "CDSERVFILI" => $servicoFilial['CDSERVFILI'],
            "NMSERVICO"  => $servicoFilial['NMSERVICO'],
            "CDGRUPSERV" => $servicoFilial['CDGRUPSERV']
          )
        );
      }
      
      return $nrlista;
    } catch (\Exception $e) {
      throw $e; 
		}
  }
  
}