<?php
namespace Planejamento\Entities;

class Recefili extends Base\Recefili
{
    /**
     * Atualiza as quantidades da receita local
     *
     * @param String $nrorg           Organização
     * @param String $cdprato         Prato
     * @param String $cdfilauxprat    Fila de prato
     * @param String $cdproduto       Produto
     * @param String $qtbrutalo       Quantidade Requerida
     * @param String $qtbrutalounr    Quantidade Bruta
     * @param String $qtcalcnutrlo    Quantidade de Cálculo de Nutrientes
     * @param String $qtcomplprodfili Quantidade Complementar
     * @param String $qtprodrecloc    Quantidade
     * @param String $qtunidestlo     Quantidade líquida do produto na unidade de estoque
     *
     * @throws \Exception
     *
     * @return null
     */
    public static function atualizaQtdeReceitaLocal(
        $produtoReceita, $qtbrutalo, $qtbrutalounr,
        $qtcalcnutrlo, $qtcomplprodfili, $qtprodrecloc, $qtunidestlo
    ) {
        try {
            $produtoReceita->setQtbrutalo($qtbrutalo);
            $produtoReceita->setQtbrutalounr($qtbrutalounr);
            $produtoReceita->setQtcalcnutrlo($qtcalcnutrlo);
            $produtoReceita->setQtcomplprodfili($qtcomplprodfili);
            $produtoReceita->setQtprodrecloc($qtprodrecloc);
            $produtoReceita->setQtunidestlo($qtunidestlo);
            $produtoReceita->persist();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna os produtos da receita local
     *
     * @param String $nrorg        Organização
     * @param String $cdprato      Prato
     * @param String $cdfilauxprat Fila de prato
     *
     * @return mixed
     * @throws \Exception
     */
    public static function getProdutosReceitaLocal($nrorg, $cdfilial, $cdprato, $cdfilauxprat)
    {
        try {
            /** @var Recefili $receitaLocal Receita Local */
            $receitaLocal = Recefili::getObjectsByWhere(
                "NRORG = ? AND CDFILIAL = ? AND CDPRATO = ? AND CDFILAUXPRAT = ?",
                array($nrorg, $cdfilial, $cdprato, $cdfilauxprat)
            );

            return $receitaLocal ;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna o produto da receita local
     *
     * @param String $nrorg        Organização
     * @param String $cdprato      Prato
     * @param String $cdfilauxprat Fila de prato
     * @param String $cdproduto    Produto
     *
     * @throws \Exception
     * @return mixed
     */
    public static function getProdutoReceitaLocal($nrorg, $cdprato,
        $cdfilauxprat, $cdproduto)
    {
        try {
            /** @var Recefili $receitaLocal Receita Local */
            $receitaLocal = Recefili::getObjectByWhere(
                'NRORG = ? AND CDPRATO = ? AND CDFILAUXPRAT = ? AND CDPRODUTO = ?',
                array($nrorg, $cdprato, $cdfilauxprat, $cdproduto)
            );

            return $receitaLocal;
        } catch (\Exception $e) {
            throw $e;
        }
    }

}