/* Tabela LANCTOESTOQ */
/* Descrio: A trigger insere registros na tabela LOGOPERAESPE a partir de alteraes na tabela LANCTOESTOQ */
/* Objetivo: Gerar um log que vai registrar as alteraes na tabela LANCTOESTOQ
             utilizando a tabela Geral de Logs de Operaes especficas*/
/* Criao:
/* 24/05/2007 - Fabrcio Marcelo Silva */


CREATE OR REPLACE TRIGGER SDXLOG_ATUESTOQ
AFTER INSERT OR UPDATE OR DELETE ON LANCTOESTOQ FOR EACH ROW
DECLARE
   V_IDUTLOGOPERESTO  VARCHAR2(1);
   V_IDTIPOOPER       VARCHAR2(1);  /* V_IDTIPOOPER - 'I' - Incluso, 'A' - Alterao, 'E' - Excluso */
   V_CLIENT_INFO      VARCHAR2(64);
   V_CDOPERADOR       VARCHAR2(12);
   V_CDFILIAL         VARCHAR2(4);
   V_DESC             VARCHAR2(500);
   V_NOMEOPERACAO     VARCHAR2(30);

BEGIN
   BEGIN
      SELECT IDUTLOGOPERESTO
      INTO V_IDUTLOGOPERESTO
      FROM PARAMGERAL
      WHERE CDPARGERAL = '0001';
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_IDUTLOGOPERESTO := 'N';
   END;

   IF V_IDUTLOGOPERESTO = 'S' THEN

      V_IDTIPOOPER  := '';
      V_CLIENT_INFO := '';
      V_CDOPERADOR  := '';
      V_CDFILIAL    := '';
      V_DESC        := '';

      /* Retorna o codigo e o nome do operador */
      DBMS_APPLICATION_INFO.READ_CLIENT_INFO(V_CLIENT_INFO);
      V_CDOPERADOR := SUBSTR(V_CLIENT_INFO,1,12);

      IF DELETING THEN

         V_DESC  := 'Filial:'         || ' ' || :OLD.CDFILIAL    || ' ' ||
                    'N Lancamento:'  || ' ' || :OLD.NRLANCESTQ  || ' ' ||
                    'Dt. Lancamento:' || ' ' || TO_CHAR(:OLD.DTLANCESTQ, 'DD/MM/YYYY');

         V_CDFILIAL   := :OLD.CDFILIAL;

         IF :OLD.IDTPLANCTO = '0' THEN
            V_NOMEOPERACAO := 'Implantacao Estoque';
         ELSE
            IF :OLD.IDTPLANCTO = '3' THEN
               V_NOMEOPERACAO := 'Retirada Estoque';
            ELSE
               IF :OLD.IDTPLANCTO = '1' THEN
                  V_NOMEOPERACAO := 'Ajuste Estoque';
               ELSE
                  V_NOMEOPERACAO := '';
               END IF;
            END IF;
         END IF;

         V_IDTIPOOPER := 'E';
      ELSE
         IF UPDATING THEN
            IF (:OLD.CDFILIAL    <> :NEW.CDFILIAL)   OR
               (:OLD.NRLANCESTQ  <> :NEW.NRLANCESTQ) OR
               (:OLD.DTLANCESTQ  <> :NEW.DTLANCESTQ) THEN

               V_IDTIPOOPER := 'A';
            END IF;
         ELSE
            V_IDTIPOOPER := 'I';
         END IF;

         V_DESC  := 'Filial:'         || ' ' || :NEW.CDFILIAL    || ' ' ||
                    'N Lancamento:'  || ' ' || :NEW.NRLANCESTQ  || ' ' ||
                    'Dt. Lancamento:' || ' ' || TO_CHAR(:NEW.DTLANCESTQ, 'DD/MM/YYYY');

         V_CDFILIAL   := :NEW.CDFILIAL;

         IF :NEW.IDTPLANCTO = '0' THEN
            V_NOMEOPERACAO := 'Implantacao Estoque';
         ELSE
            IF :NEW.IDTPLANCTO = '3' THEN
               V_NOMEOPERACAO := 'Retirada Estoque';
            ELSE
               IF :NEW.IDTPLANCTO = '1' THEN
                  V_NOMEOPERACAO := 'Ajuste Estoque';
               ELSE
                  V_NOMEOPERACAO := '';
               END IF;
            END IF;
         END IF;

      END IF;

      IF NOT (V_DESC IS NULL)         AND
         NOT (V_IDTIPOOPER IS NULL)   AND
         NOT (V_NOMEOPERACAO IS NULL) THEN

         V_DESC := SUBSTR(V_DESC, 1, 500);

         INSERT INTO LOGOPERAESTO (NMOPERAESTO,     DTOPERAESTO,     IDOPERAESTO,
                                   CDOPERRESPOPEST, DSOPERAESTO, CDFILIRESPOPEST)
                     VALUES       (V_NOMEOPERACAO,  SYSDATE,         V_IDTIPOOPER,
                                   V_CDOPERADOR,    V_DESC,      V_CDFILIAL);
      END IF;
   END IF;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('SDXLOG_ATUESTOQ','5.05.021'); END;
/

