import {
  ModalService, Modal, Form, Select, Radio, Number as NumberInput, DialogService, Grid,
} from '@zeedhi/common';
import {
  Metadata, IDictionary, Loader, Http, IEventParam,
} from '@zeedhi/core';

export class AlteracaoReceitas {
  public modalAteracaoReceitas!: Modal;
  
  public opcao = '';
  
  public forma = '';
  
  public utilities = Loader.getInstance('Utilities');
  
  public receitaController = Loader.getInstance('ReceitaController');
  
  private async createModalAteracaoReceitas() {
    this.modalAteracaoReceitas = await ModalService.createFromJson(
      'planejamento/prato/modals/alteracao_receitas', true,
    );
  }
  
  public closeModalAteracaoReceitas() {
    const grid: Grid = Metadata.getInstance('gridUtilizacaoProd');
    grid.selectedRows = [];
    this.modalAteracaoReceitas.hide();
    this.modalAteracaoReceitas.destroy();
    this.modalAteracaoReceitas = this.utilities.modalEmpty;
  }
  
  public async openModalAteracaoReceitas(opcao: string) {
    this.opcao = opcao;
    if (!this.modalAteracaoReceitas) {
      await this.createModalAteracaoReceitas().then(() => {
        this.modalAteracaoReceitas.show();
      });
    } else {
      await this.modalAteracaoReceitas.show();
    }
  }
  
  public setInitFields() {
    const prod = Metadata.getInstance<Select>('PRODUTO');
    const un = Metadata.getInstance<Select>('SGUNIDADE');
    const form = Metadata.getInstance<Form>('formAlteracaoReceitas');
    const qtProdPrat = form.getChildInstance<NumberInput>('QTPRODPRAT');
    const qtBruta = form.getChildInstance<NumberInput>('QTBRUTAPRUNR');
    const ganho = form.getChildInstance<NumberInput>('VRPEFATCOCPR');
    const apCoccao = form.getChildInstance<NumberInput>('VRPEFATCOCPRAP');
    const perdaLimpeza = form.getChildInstance<NumberInput>('VRPEFATCORPR');
    const apLimpeza = form.getChildInstance<NumberInput>('VRPEFATCORPRAP');
    const hora = form.getChildInstance<NumberInput>('HRTEMPPREPPR');
    const apNutricional = form.getChildInstance<NumberInput>('VRPEFATNUTPR');
    const cdprod = Metadata.getInstance<Select>('CDPRODUTO');
    let qtAtiva: NumberInput;
    if (this.receitaController.idqtliqubrut) {
      qtProdPrat.disabled = true;
      qtBruta.disabled = false;
      qtAtiva = qtBruta;
    } else {
      qtProdPrat.disabled = false;
      qtBruta.disabled = true;
      qtAtiva = qtProdPrat;
    }
    
    // if (cdprod.value) this.opcao = 'A';
    if (this.opcao === 'A') {
      Metadata.getInstance<Form>('formAlteracaoReceitas').value.OPCAO = 'A';
      const forma = Metadata.getInstance<Form>('formFiltroUtilizacaoProd').value.FORMA;
      Metadata.getInstance<Form>('formAlteracaoReceitas').value.FORMA = forma;
      if (cdprod.value && cdprod.value.length >= 10) {
        prod.disabled = false;
        un.disabled = false;
      } else {
        prod.disabled = true;
        un.disabled = true;
      }
      prod.removeValidation('required');
      prod.setValue('');
      // un.removeValidation('required');
      un.setValue('');
      qtAtiva.removeValidation('required');
      qtAtiva.value = null;
      ganho.removeValidation('required');
      ganho.value = null;
      apCoccao.removeValidation('required');
      apCoccao.value = null;
      perdaLimpeza.removeValidation('required');
      perdaLimpeza.value = null;
      apLimpeza.removeValidation('required');
      apLimpeza.value = null;
      hora.removeValidation('required');
      hora.value = null;
      apNutricional.removeValidation('required');
      apNutricional.value = null;
    } else if (this.opcao === 'I') {
      // setar forma como quantidade
      Metadata.getInstance<Form>('formAlteracaoReceitas').value.OPCAO = 'I';
      // this.forma = 'Q';
      Metadata.getInstance<Radio>('FORMA').value = 'Q';
      prod.disabled = false;
      prod.addValidation('required', { message: 'pme::campoObrigatorio' });
      prod.setValue('');
      un.disabled = false;
      un.setValue('');
      qtAtiva.addValidation('required', { message: 'pme::campoObrigatorio' });
      qtAtiva.value = null;
      ganho.addValidation('required', { message: 'pme::campoObrigatorio' });
      ganho.value = 0;
      apCoccao.addValidation('required', { message: 'pme::campoObrigatorio' });
      apCoccao.value = 100;
      perdaLimpeza.addValidation('required', { message: 'pme::campoObrigatorio' });
      perdaLimpeza.value = 0;
      apLimpeza.addValidation('required', { message: 'pme::campoObrigatorio' });
      apLimpeza.value = 100;
      hora.addValidation('required', { message: 'pme::campoObrigatorio' });
      hora.value = null;
      apNutricional.addValidation('required', { message: 'pme::campoObrigatorio' });
      apNutricional.value = 100;
    }
    
    if (Metadata.getInstance<Radio>('FORMA').value === 'P') {
      form.getChildInstance<NumberInput>('QTPRODPRAT').label = 'pme::quantidadePerc';
      form.getChildInstance<NumberInput>('QTBRUTAPRUNR').label = 'pme::qtBrutaPerc';
    }
  }
  
  public filterSGUN({ component }: IDictionary) {
    const prod = Metadata.getInstance<Select>('PRODUTO');
    const filter = [
      {
        name: 'CDPRODUTO',
        operator: '=',
        value: prod.selectValue.CDPRODUTO || null,
      },
      {
        name: 'SGUNIDADE',
        operator: '=',
        value: prod.selectValue.SGUNIDADE || null,
      },
      {
        name: 'SGUNIDESTO',
        operator: '=',
        value: null,
      },
      {
        name: 'TSGUNIDADE',
        operator: '=',
        value: 'T',
      },
    ];
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public validFields() {
    const formValue = Metadata.getInstance<Form>('formAlteracaoReceitas').value;
    // const qtBruta = Metadata.getInstance<NumberInput>('QTBRUTAPRUNR');
    const ganhoCoccao = Metadata.getInstance<NumberInput>('VRPEFATCOCPR');
    const apCoccao = Metadata.getInstance<NumberInput>('VRPEFATCOCPRAP');
    const perdaLimpeza = Metadata.getInstance<NumberInput>('VRPEFATCORPR');
    const apLimpeza = Metadata.getInstance<NumberInput>('VRPEFATCORPRAP');
    const apNutricional = Metadata.getInstance<NumberInput>('VRPEFATNUTPR');

    // if ((formValue.QTBRUTAPRUNR === undefined) && (this.opcao === 'A')) {
    //   qtBruta.addValidation('required', { message: 'pme::campoObrigatorio' });
    // }
    
    if (formValue.QTBRUTAPRUNR <= 0) {
      this.utilities.showAlert('alert', 'pme::minQtdeBruta', 'warning');
      return false;
    }
    
    // ganho e aprovação de coccao
    if ((formValue.VRPEFATCOCPR !== undefined || formValue.VRPEFATCOCPRAP !== undefined) /* && this.opcao === 'A' */) {
      ganhoCoccao.addValidation('required', { message: 'pme::campoObrigatorio' });
      apCoccao.addValidation('required', { message: 'pme::campoObrigatorio' });
      
      if (formValue.VRPEFATCOCPR < -99.99) {
        this.utilities.showAlert('alert', 'pme::minGanhoCoccao', 'warning');
        return false;
      }
      
      if (formValue.VRPEFATCOCPRAP < 0) {
        this.utilities.showAlert('alert', 'pme::minApCoccao', 'warning');
        return false;
      }
    }
    
    // perda e aprovacao limpeza
    if ((formValue.VRPEFATCORPR !== undefined || formValue.VRPEFATCORPRAP !== undefined) /* && (this.opcao === 'A') */) {
      perdaLimpeza.addValidation('required', { message: 'pme::campoObrigatorio' });
      apLimpeza.addValidation('required', { message: 'pme::campoObrigatorio' });
      
      if (formValue.VRPEFATCORPR < 0) {
        this.utilities.showAlert('alert', 'pme::minPerdaLimp', 'warning');
        return false;
      }
      
      if (formValue.VRPEFATCORPR > 99) {
        this.utilities.showAlert('alert', 'pme::maxPerdaLimp', 'warning');
        return false;
      }
    
      if (formValue.VRPEFATCORPRAP < 1) {
        this.utilities.showAlert('alert', 'pme::minApLimp', 'warning');
        return false;
      }
      
      if (formValue.VRPEFATCORPRAP > 100) {
        this.utilities.showAlert('alert', 'pme::maxApLimp', 'warning');
        return false;
      }
    }
    
    // degelo
    if (formValue.VRPERCDEGELO < 0) {
      this.utilities.showAlert('alert', 'pme::minDeGelo', 'warning');
      return false;
    }
    if (formValue.VRPERCDEGELO > 99) {
      this.utilities.showAlert('alert', 'pme::maxDeGelo', 'warning');
      return false;
    }
    
    if (formValue.VRPEFATNUTPR === null || formValue.VRPEFATNUTPR === undefined) {
      if (this.opcao === 'I') {
        apNutricional.addValidation('required', { message: 'pme::campoObrigatorio' });
        return false;
      }
    }
    
    if (formValue.VRPEFATNUTPR !== undefined && formValue.VRPEFATNUTPR <= 0) {
      this.utilities.showAlert('alert', 'pme::minAprovNutri', 'warning');
      return false;
    }
    
    return true;
  }
  
  public confirmar() {
    const form = Metadata.getInstance<Form>('formAlteracaoReceitas');
    const validFields = this.validFields();
    
    if (form.validate()) {
      if (validFields) {
        let algumValorPreenchido = false;
        const values = form.value;
        Object.keys(values).forEach((key: string) => {
          if (key !== 'OPCAO' && key !== 'PRODUTO' && key !== 'SGUNIDADE' && key !== 'FORMA') {
            const field = Metadata.getInstance<NumberInput>(key);
            if (field.value && field.disabled === false) {
              algumValorPreenchido = true;
            }
          }
        });
        const cdprod = Metadata.getInstance<Select>('CDPRODUTO');
        const prod = Metadata.getInstance<Select>('PRODUTO');
        if (algumValorPreenchido) {
          if (cdprod.value && cdprod.value.length >= 10 && prod.value && this.opcao === 'A') {
            DialogService.show({
              name: 'confirmaSubsProduto',
              type: 'warning',
              title: 'pme::atencao',
              text: 'pme::confirmaSubsProduto',
              buttons: [
                {
                  name: 'salvar',
                  component: 'ZdButton',
                  label: 'pme::sim',
                  events: { click: () => this.sim() },
                  outline: true,
                },
                {
                  name: 'no',
                  component: 'ZdButton',
                  label: 'pme::nao',
                  events: { click: () => DialogService.hide() },
                },
                {
                  name: 'progressLine',
                  component: 'ZdProgress',
                  label: 'Progress',
                  indeterminate: true,
                  cssClass: 'loading-progress-line',
                },
              ],
            });
          } else {
            DialogService.show({
              name: 'confirmaAlteracaoReceita',
              type: 'warning',
              title: 'pme::atencao',
              text: 'pme::confirmaAlteracaoReceita',
              buttons: [
                {
                  name: 'salvar',
                  component: 'ZdButton',
                  label: 'pme::sim',
                  events: { click: () => this.sim() },
                  outline: true,
                },
                {
                  name: 'no',
                  component: 'ZdButton',
                  label: 'pme::nao',
                  events: { click: () => DialogService.hide() },
                },
                {
                  name: 'progressLine',
                  component: 'ZdProgress',
                  label: 'Progress',
                  indeterminate: true,
                  cssClass: 'loading-progress-line',
                },
              ],
            });
          }
        } else {
          DialogService.show({
            name: 'nenhumAtribudoPreenchido',
            type: 'warning',
            title: 'pme::atencao',
            text: 'pme::nenhumAtribudoPreenchido',
            buttons: [
              {
                name: 'ok',
                component: 'ZdButton',
                label: 'pme::OK',
                events: { click: () => DialogService.hide() },
              },
            ],
          });
        }
      }
    }
  }
  
  public sim() {
    DialogService.hide();
    const grid: Grid = Metadata.getInstance('gridUtilizacaoProd');
    const rows = grid.selectedRows;
    const form = Metadata.getInstance<Form>('formAlteracaoReceitas');
    const prod = Metadata.getInstance<Select>('PRODUTO');
    const sg = Metadata.getInstance<Select>('SGUNIDADE');
    // HRTEMPPREPPR = hora.replace(':', '').split(' ')[0];
    const qtRequerida = form.value.QTBRUTAPRUNR && sg.selectValue
      ? form.value.QTBRUTAPRUNR * sg.selectValue.VRFATOCONVES
      : form.value.QTBRUTAPRUNR;
    const values = {
      CDPRODUTO: form.value.PRODUTO ? form.value.PRODUTO : null,
      SGUNIDADE: form.value.SGUNIDADE ? form.value.SGUNIDADE : null,
      QTPRODPRAT: form.value.QTPRODPRAT ? form.value.QTPRODPRAT : null,
      QTBRUTAPRUNR: form.value.QTBRUTAPRUNR ? form.value.QTBRUTAPRUNR : null,
      QTBRUTA: qtRequerida,
      VRPEFATCOCPR: form.value.VRPEFATCOCPR ? form.value.VRPEFATCOCPR : null,
      VRPEFATCORPR: form.value.VRPEFATCORPR ? form.value.VRPEFATCORPR : null,
      HRTEMPPREPPR: form.value.HRTEMPPREPPR ? form.value.HRTEMPPREPPR.replace(':', '').split(' ')[0] : null,
      VRPEFATCOCPRAP: form.value.VRPEFATCOCPRAP ? form.value.VRPEFATCOCPRAP : null,
      VRPEFATCORPRAP: form.value.VRPEFATCORPRAP ? form.value.VRPEFATCORPRAP : null,
      VRPEFATNUTPR: form.value.VRPEFATNUTPR ? form.value.VRPEFATNUTPR : null,
      VRPERCDEGELO: form.value.VRPERCDEGELO ? form.value.VRPERCDEGELO : null,
      FORMA: form.value.FORMA ? form.value.FORMA : null,
      OPCAO: form.value.OPCAO ? form.value.OPCAO : null,
      VRFATOCONVES: sg.selectValue ? sg.selectValue.VRFATOCONVES : null,
      NMPRODUTO: prod.selectValue ? prod.selectValue.NMPRODUTO : null,
    };
    let qtAtiva: string;
    if (this.receitaController.idqtliqubrut) {
      qtAtiva = 'L';
    } else qtAtiva = 'B';
    Http.post('/alteracaoAutomatica', {
      requestType: 'Row',
      produtos: rows,
      values,
      QTATIVA: qtAtiva,
    }).then((response: IDictionary) => {
      if (response.data) {
        if (response.data.dataset) {
          if (response.data.dataset.RETORNO[0]) {
            this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
          }
        } else this.utilities.showAlert('alert', `pme::${response.data.error}`, 'error');
      }
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    }).finally(() => {
      // this.utilities.disableLoadingButton(component);
      grid.selectedRows = [];
      grid.reload();
      this.closeModalAteracaoReceitas();
      // DialogService.hide();
    });
  }
  
  public trataQuantidade({ component }: IEventParam<NumberInput>) {
    const form = Metadata.getInstance<Form>('formAlteracaoReceitas');
    switch (component.name) {
      case 'VRPEFATCOCPR':
        if (component.value || component.value === 0 || form.value.VRPEFATCOCPRAP) {
          form.value.VRPEFATCOCPRAP = 100 + (component.value || 0);
        }
        break;
      case 'VRPEFATCOCPRAP':
        if (component.value || component.value === 0 || form.value.VRPEFATCOCPR) {
          form.value.VRPEFATCOCPR = (component.value || 0) - 100;
        }
        break;
      case 'VRPEFATCORPR':
        if (component.value || component.value === 0 || form.value.VRPEFATCORPRAP) {
          form.value.VRPEFATCORPRAP = 100 - (component.value || 0);
        }
        break;
      case 'VRPEFATCORPRAP':
        if (component.value || component.value === 0 || form.value.VRPEFATCORPR) {
          form.value.VRPEFATCORPR = 100 - (component.value || 0);
        }
        break;
      default:
        break;
    }
    
    if (component.value === undefined && this.opcao === 'I') component.removeValidation('required');
  }
  
  public validateUnity({ component }: IEventParam<Select>) {
    const un = Metadata.getInstance<Select>('SGUNIDADE');
    if (this.opcao === 'A' && component.selectValue) {
      un.addValidation('required', { message: 'pme::campoObrigatorio' });
      un.disabled = false;
      un.setValue('');
    } else if (this.opcao === 'A') {
      un.removeValidation('required');
      un.disabled = true;
      un.setValue('');
    }
  }
}
